@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'parser/ruby18'
 require 'parser/ruby19'
 require 'parser/ruby20'
@@ -6,3 +8,5 @@ require 'parser/ruby22'
 require 'parser/ruby23'
 require 'parser/ruby24'
 require 'parser/ruby25'
+require 'parser/ruby26'
+require 'parser/ruby27'
