@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module Parser
 
   ##
@@ -29,10 +31,10 @@ module Parser
       # all new code should set this attribute to true.
       #
       # If set to false (the default), arguments of `m { |a| }` are emitted as
-      # `s(:args, s(:arg, :a))`
+      # `s(:args, s(:arg, :a))`.
       #
       # If set to true, arguments of `m { |a| }` are emitted as
-      # `s(:args, s(:procarg0, :a))
+      # `s(:args, s(:procarg0, :a)).
       #
       # @return [Boolean]
       attr_accessor :emit_procarg0
@@ -40,12 +42,70 @@ module Parser
 
     @emit_procarg0 = false
 
+    class << self
+      ##
+      # AST compatibility attribute; locations of `__ENCODING__` are not the same
+      # as locations of `Encoding::UTF_8` causing problems during rewriting,
+      # all new code should set this attribute to true.
+      #
+      # If set to false (the default), `__ENCODING__` is emitted as
+      # ` s(:const, s(:const, nil, :Encoding), :UTF_8)`.
+      #
+      # If set to true, `__ENCODING__` is emitted as
+      # `s(:__ENCODING__)`.
+      #
+      # @return [Boolean]
+      attr_accessor :emit_encoding
+    end
+
+    @emit_encoding = false
+
+    class << self
+      ##
+      # AST compatibility attribute; indexed assignment, `x[] = 1`, is not
+      # semantically equivalent to calling the method directly, `x.[]=(1)`.
+      # Specifically, in the former case, the expression's value is always 1,
+      # and in the latter case, the expression's value is the return value
+      # of the `[]=` method.
+      #
+      # If set to false (the default), `self[1]` is emitted as
+      # `s(:send, s(:self), :[], s(:int, 1))`, and `self[1] = 2` is
+      # emitted as `s(:send, s(:self), :[]=, s(:int, 1), s(:int, 2))`.
+      #
+      # If set to true, `self[1]` is emitted as
+      # `s(:index, s(:self), s(:int, 1))`, and `self[1] = 2` is
+      # emitted as `s(:indexasgn, s(:self), s(:int, 1), s(:int, 2))`.
+      #
+      # @return [Boolean]
+      attr_accessor :emit_index
+    end
+
+    class << self
+      ##
+      # AST compatibility attribute; causes a single non-mlhs
+      # block argument to be wrapped in s(:procarg0).
+      #
+      # If set to false (the default), block arguments `|a|` are emitted as
+      # `s(:args, s(:procarg0, :a))`
+      #
+      # If set to true, block arguments `|a|` are emitted as
+      # `s(:args, s(:procarg0, s(:arg, :a))`
+      #
+      # @return [Boolean]
+      attr_accessor :emit_arg_inside_procarg0
+    end
+
+    @emit_index = false
+
     class << self
       ##
       # @api private
       def modernize
         @emit_lambda = true
         @emit_procarg0 = true
+        @emit_encoding = true
+        @emit_index = true
+        @emit_arg_inside_procarg0 = true
       end
     end
 
@@ -54,7 +114,7 @@ module Parser
     attr_accessor :parser
 
     ##
-    # If set to true, `__FILE__` and `__LINE__` are transformed to
+    # If set to true (the default), `__FILE__` and `__LINE__` are transformed to
     # literal nodes. For example, `s(:str, "lib/foo.rb")` and `s(:int, 10)`.
     #
     # If set to false, `__FILE__` and `__LINE__` are emitted as-is,
@@ -356,12 +416,12 @@ module Parser
 
     def range_inclusive(lhs, dot2_t, rhs)
       n(:irange, [ lhs, rhs ],
-        binary_op_map(lhs, dot2_t, rhs))
+        range_map(lhs, dot2_t, rhs))
     end
 
     def range_exclusive(lhs, dot3_t, rhs)
       n(:erange, [ lhs, rhs ],
-        binary_op_map(lhs, dot3_t, rhs))
+        range_map(lhs, dot3_t, rhs))
     end
 
     #
@@ -422,13 +482,22 @@ module Parser
         end
 
       when :__ENCODING__
-        n(:const, [ n(:const, [ nil, :Encoding], nil), :UTF_8 ],
-          node.loc.dup)
+        if !self.class.emit_encoding
+          n(:const, [ n(:const, [ nil, :Encoding], nil), :UTF_8 ],
+            node.loc.dup)
+        else
+          node
+        end
 
       when :ident
         name, = *node
 
         if @parser.static_env.declared?(name)
+          if name.to_s == parser.current_arg_stack.top
+            diagnostic :error, :circular_argument_reference,
+                       { :var_name => name.to_s }, node.loc.expression
+          end
+
           node.updated(:lvar)
         else
           name, = *node
@@ -479,7 +548,7 @@ module Parser
         node.updated(:gvasgn)
 
       when :const
-        if @parser.in_def?
+        unless @parser.context.dynamic_const_definition_allowed?
           diagnostic :error, :dynamic_const, nil, node.loc.expression
         end
 
@@ -487,6 +556,9 @@ module Parser
 
       when :ident
         name, = *node
+
+        check_assignment_to_numparam(node)
+
         @parser.static_env.declare(name)
 
         node.updated(:lvasgn)
@@ -513,12 +585,16 @@ module Parser
 
     def op_assign(lhs, op_t, rhs)
       case lhs.type
-      when :gvasgn, :ivasgn, :lvasgn, :cvasgn, :casgn, :send, :csend
+      when :gvasgn, :ivasgn, :lvasgn, :cvasgn, :casgn, :send, :csend, :index
         operator   = value(op_t)[0..-1].to_sym
         source_map = lhs.loc.
                         with_operator(loc(op_t)).
                         with_expression(join_exprs(lhs, rhs))
 
+        if lhs.type  == :index
+          lhs = lhs.updated(:indexasgn)
+        end
+
         case operator
         when :'&&'
           n(:and_asgn, [ lhs, rhs ], source_map)
@@ -611,6 +687,14 @@ module Parser
         collection_map(begin_t, args, end_t))
     end
 
+    def numargs(max_numparam)
+      n(:numargs, [ max_numparam ], nil)
+    end
+
+    def forward_args(begin_t, dots_t, end_t)
+      n(:forward_args, [], collection_map(begin_t, token_map(dots_t), end_t))
+    end
+
     def arg(name_t)
       n(:arg, [ value(name_t).to_sym ],
         variable_map(name_t))
@@ -653,6 +737,11 @@ module Parser
       end
     end
 
+    def kwnilarg(dstar_t, nil_t)
+      n0(:kwnilarg,
+        arg_prefix_map(dstar_t, nil_t))
+    end
+
     def shadowarg(name_t)
       n(:shadowarg, [ value(name_t).to_sym ],
         variable_map(name_t))
@@ -665,7 +754,12 @@ module Parser
 
     def procarg0(arg)
       if self.class.emit_procarg0
-        arg.updated(:procarg0)
+        if arg.type == :arg && self.class.emit_arg_inside_procarg0
+          n(:procarg0, [ arg ],
+            Source::Map::Collection.new(nil, nil, arg.location.expression))
+        else
+          arg.updated(:procarg0)
+        end
       else
         arg
       end
@@ -750,6 +844,10 @@ module Parser
       end
     end
 
+    def forwarded_args(dots_t)
+      n(:forwarded_args, [], token_map(dots_t))
+    end
+
     def call_method(receiver, dot_t, selector_t,
                     lparen_t=nil, args=[], rparen_t=nil)
       type = call_type_for_dot(dot_t)
@@ -779,19 +877,26 @@ module Parser
       end
 
       last_arg = call_args.last
-      if last_arg && last_arg.type == :block_pass
+      if last_arg && (last_arg.type == :block_pass || last_arg.type == :forwarded_args)
         diagnostic :error, :block_and_blockarg, nil, last_arg.loc.expression, [loc(begin_t)]
       end
 
-      if [:send, :csend, :super, :zsuper, :lambda].include?(method_call.type)
-        n(:block, [ method_call, args, body ],
+      if args.type == :numargs
+        block_type = :numblock
+        args = args.children[0]
+      else
+        block_type = :block
+      end
+
+      if [:send, :csend, :index, :super, :zsuper, :lambda].include?(method_call.type)
+        n(block_type, [ method_call, args, body ],
           block_map(method_call.loc.expression, begin_t, end_t))
       else
         # Code like "return foo 1 do end" is reduced in a weird sequence.
         # Here, method_call is actually (return).
         actual_send, = *method_call
         block =
-          n(:block, [ actual_send, args, body ],
+          n(block_type, [ actual_send, args, body ],
             block_map(actual_send.loc.expression, begin_t, end_t))
 
         n(method_call.type, [ block ],
@@ -823,14 +928,24 @@ module Parser
     end
 
     def index(receiver, lbrack_t, indexes, rbrack_t)
-      n(:send, [ receiver, :[], *indexes ],
-        send_index_map(receiver, lbrack_t, rbrack_t))
+      if self.class.emit_index
+        n(:index, [ receiver, *indexes ],
+          index_map(receiver, lbrack_t, rbrack_t))
+      else
+        n(:send, [ receiver, :[], *indexes ],
+          send_index_map(receiver, lbrack_t, rbrack_t))
+      end
     end
 
     def index_asgn(receiver, lbrack_t, indexes, rbrack_t)
-      # Incomplete method call.
-      n(:send, [ receiver, :[]=, *indexes ],
-        send_index_map(receiver, lbrack_t, rbrack_t))
+      if self.class.emit_index
+        n(:indexasgn, [ receiver, *indexes ],
+          index_map(receiver, lbrack_t, rbrack_t))
+      else
+        # Incomplete method call.
+        n(:send, [ receiver, :[]=, *indexes ],
+          send_index_map(receiver, lbrack_t, rbrack_t))
+      end
     end
 
     def binary_op(receiver, operator_t, arg)
@@ -1093,6 +1208,188 @@ module Parser
       end
     end
 
+    #
+    # PATTERN MATCHING
+    #
+
+    def case_match(case_t, expr, in_bodies, else_t, else_body, end_t)
+      n(:case_match, [ expr, *(in_bodies << else_body)],
+        condition_map(case_t, expr, nil, nil, else_t, else_body, end_t))
+    end
+
+    def in_match(lhs, in_t, rhs)
+      n(:in_match, [lhs, rhs],
+        binary_op_map(lhs, in_t, rhs))
+    end
+
+    def in_pattern(in_t, pattern, guard, then_t, body)
+      children = [pattern, guard, body]
+      n(:in_pattern, children,
+        keyword_map(in_t, then_t, children.compact, nil))
+    end
+
+    def if_guard(if_t, if_body)
+      n(:if_guard, [if_body], guard_map(if_t, if_body))
+    end
+
+    def unless_guard(unless_t, unless_body)
+      n(:unless_guard, [unless_body], guard_map(unless_t, unless_body))
+    end
+
+    def match_var(name_t)
+      name = value(name_t).to_sym
+
+      check_duplicate_pattern_variable(name, loc(name_t))
+      @parser.static_env.declare(name)
+
+      n(:match_var, [ name ],
+        variable_map(name_t))
+    end
+
+    def match_hash_var(name_t)
+      name = value(name_t).to_sym
+
+      expr_l = loc(name_t)
+      name_l = expr_l.adjust(end_pos: -1)
+
+      check_duplicate_pattern_variable(name, name_l)
+      @parser.static_env.declare(name)
+
+      n(:match_var, [ name ],
+        Source::Map::Variable.new(name_l, expr_l))
+    end
+
+    def match_hash_var_from_str(begin_t, strings, end_t)
+      if strings.length > 1
+        diagnostic :error, :pm_interp_in_var_name, nil, loc(begin_t).join(loc(end_t))
+      end
+
+      string = strings[0]
+
+      case string.type
+      when :str
+        # MRI supports plain strings in hash pattern matching
+        name, = *string
+        name_l = string.loc.expression
+
+        check_lvar_name(name, name_l)
+        check_duplicate_pattern_variable(name, name_l)
+
+        @parser.static_env.declare(name)
+
+        if (begin_l = string.loc.begin)
+          # exclude beginning of the string from the location of the variable
+          name_l = name_l.adjust(begin_pos: begin_l.length)
+        end
+
+        if (end_l = string.loc.end)
+          # exclude end of the string from the location of the variable
+          name_l = name_l.adjust(end_pos: -end_l.length)
+        end
+
+        expr_l = loc(begin_t).join(string.loc.expression).join(loc(end_t))
+        n(:match_var, [ name.to_sym ],
+          Source::Map::Variable.new(name_l, expr_l))
+      when :begin
+        match_hash_var_from_str(begin_t, string.children, end_t)
+      end
+    end
+
+    def match_rest(star_t, name_t = nil)
+      if name_t.nil?
+        n0(:match_rest,
+          unary_op_map(star_t))
+      else
+        name = match_var(name_t)
+        n(:match_rest, [ name ],
+          unary_op_map(star_t, name))
+      end
+    end
+
+    def hash_pattern(lbrace_t, kwargs, rbrace_t)
+      args = check_duplicate_args(kwargs)
+      n(:hash_pattern, args,
+        collection_map(lbrace_t, args, rbrace_t))
+    end
+
+    def array_pattern(lbrack_t, elements, rbrack_t)
+      trailing_comma = false
+
+      elements = elements.map do |element|
+        if element.type == :match_with_trailing_comma
+          trailing_comma = true
+          element.children.first
+        else
+          trailing_comma = false
+          element
+        end
+      end
+
+      node_type = trailing_comma ? :array_pattern_with_tail : :array_pattern
+      n(node_type, elements,
+        collection_map(lbrack_t, elements, rbrack_t))
+    end
+
+    def match_with_trailing_comma(match)
+      n(:match_with_trailing_comma, [ match ], nil)
+    end
+
+    def const_pattern(const, ldelim_t, pattern, rdelim_t)
+      n(:const_pattern, [const, pattern],
+        collection_map(ldelim_t, [pattern], rdelim_t))
+    end
+
+    def pin(pin_t, var)
+      n(:pin, [ var ],
+        send_unary_op_map(pin_t, var))
+    end
+
+    def match_alt(left, pipe_t, right)
+      source_map = binary_op_map(left, pipe_t, right)
+
+      n(:match_alt, [ left, right ],
+        source_map)
+    end
+
+    def match_as(value, assoc_t, as)
+      source_map = binary_op_map(value, assoc_t, as)
+
+      n(:match_as, [ value, as ],
+        source_map)
+    end
+
+    def match_nil_pattern(dstar_t, nil_t)
+      n0(:match_nil_pattern,
+        arg_prefix_map(dstar_t, nil_t))
+    end
+
+    def match_pair(label_type, label, value)
+      if label_type == :label
+        check_duplicate_pattern_key(label[0], label[1])
+        pair_keyword(label, value)
+      else
+        begin_t, parts, end_t = label
+
+        # quoted label like "label": value
+        if (var_name = static_string(parts))
+          loc = loc(begin_t).join(loc(end_t))
+          check_duplicate_pattern_key(var_name, loc)
+        end
+
+        pair_quoted(begin_t, parts, end_t, value)
+      end
+    end
+
+    def match_label(label_type, label)
+      if label_type == :label
+        match_hash_var(label)
+      else
+        # quoted label like "label": value
+        begin_t, strings, end_t = label
+        match_hash_var_from_str(begin_t, strings, end_t)
+      end
+    end
+
     private
 
     #
@@ -1148,18 +1445,18 @@ module Parser
         case this_arg.type
         when :arg, :optarg, :restarg, :blockarg,
              :kwarg, :kwoptarg, :kwrestarg,
-             :shadowarg, :procarg0
+             :shadowarg
 
-          this_name, = *this_arg
+          check_duplicate_arg(this_arg, map)
 
-          that_arg   = map[this_name]
-          that_name, = *that_arg
+        when :procarg0
 
-          if that_arg.nil?
-            map[this_name] = this_arg
-          elsif arg_name_collides?(this_name, that_name)
-            diagnostic :error, :duplicate_argument, nil,
-                       this_arg.loc.name, [ that_arg.loc.name ]
+          if this_arg.children[0].is_a?(Symbol)
+            # s(:procarg0, :a)
+            check_duplicate_arg(this_arg, map)
+          else
+            # s(:procarg0, s(:arg, :a), ...)
+            check_duplicate_args(this_arg.children, map)
           end
 
         when :mlhs
@@ -1168,6 +1465,33 @@ module Parser
       end
     end
 
+    def check_duplicate_arg(this_arg, map={})
+      this_name, = *this_arg
+
+      that_arg   = map[this_name]
+      that_name, = *that_arg
+
+      if that_arg.nil?
+        map[this_name] = this_arg
+      elsif arg_name_collides?(this_name, that_name)
+        diagnostic :error, :duplicate_argument, nil,
+                   this_arg.loc.name, [ that_arg.loc.name ]
+      end
+    end
+
+    def check_assignment_to_numparam(node)
+      name = node.children[0].to_s
+
+      assigning_to_numparam =
+        @parser.context.in_dynamic_block? &&
+        name =~ /\A_([1-9])\z/ &&
+        @parser.max_numparam_stack.has_numparams?
+
+      if assigning_to_numparam
+        diagnostic :error, :cant_assign_to_numparam, { :name => name }, node.loc.expression
+      end
+    end
+
     def arg_name_collides?(this_name, that_name)
       case @parser.version
       when 18
@@ -1183,6 +1507,32 @@ module Parser
       end
     end
 
+    def check_lvar_name(name, loc)
+      if name =~ /\A[[[:lower:]]|_][[[:alnum:]]_]*\z/
+        # OK
+      else
+        diagnostic :error, :lvar_name, { name: name }, loc
+      end
+    end
+
+    def check_duplicate_pattern_variable(name, loc)
+      return if name.to_s.start_with?('_')
+
+      if @parser.pattern_variables.declared?(name)
+        diagnostic :error, :duplicate_variable_name, { name: name.to_s }, loc
+      end
+
+      @parser.pattern_variables.declare(name)
+    end
+
+    def check_duplicate_pattern_key(name, loc)
+      if @parser.pattern_hash_keys.declared?(name)
+        diagnostic :error, :duplicate_pattern_key, { name: name.to_s }, loc
+      end
+
+      @parser.pattern_hash_keys.declare(name)
+    end
+
     #
     # SOURCE MAPS
     #
@@ -1207,13 +1557,9 @@ module Parser
     def delimited_string_map(string_t)
       str_range = loc(string_t)
 
-      begin_l = Source::Range.new(str_range.source_buffer,
-                                  str_range.begin_pos,
-                                  str_range.begin_pos + 1)
+      begin_l = str_range.with(end_pos: str_range.begin_pos + 1)
 
-      end_l   = Source::Range.new(str_range.source_buffer,
-                                  str_range.end_pos - 1,
-                                  str_range.end_pos)
+      end_l   = str_range.with(begin_pos: str_range.end_pos - 1)
 
       Source::Map::Collection.new(begin_l, end_l,
                                   loc(string_t))
@@ -1222,9 +1568,7 @@ module Parser
     def prefix_string_map(symbol)
       str_range = loc(symbol)
 
-      begin_l = Source::Range.new(str_range.source_buffer,
-                                  str_range.begin_pos,
-                                  str_range.begin_pos + 1)
+      begin_l = str_range.with(end_pos: str_range.begin_pos + 1)
 
       Source::Map::Collection.new(begin_l, nil,
                                   loc(symbol))
@@ -1238,13 +1582,9 @@ module Parser
     def pair_keyword_map(key_t, value_e)
       key_range = loc(key_t)
 
-      key_l   = Source::Range.new(key_range.source_buffer,
-                                  key_range.begin_pos,
-                                  key_range.end_pos - 1)
+      key_l   = key_range.adjust(end_pos: -1)
 
-      colon_l = Source::Range.new(key_range.source_buffer,
-                                  key_range.end_pos - 1,
-                                  key_range.end_pos)
+      colon_l = key_range.with(begin_pos: key_range.end_pos - 1)
 
       [ # key map
         Source::Map::Collection.new(nil, nil,
@@ -1257,13 +1597,10 @@ module Parser
     def pair_quoted_map(begin_t, end_t, value_e)
       end_l = loc(end_t)
 
-      quote_l = Source::Range.new(end_l.source_buffer,
-                                  end_l.end_pos - 2,
-                                  end_l.end_pos - 1)
+      quote_l = end_l.with(begin_pos: end_l.end_pos - 2,
+                           end_pos: end_l.end_pos - 1)
 
-      colon_l = Source::Range.new(end_l.source_buffer,
-                                  end_l.end_pos - 1,
-                                  end_l.end_pos)
+      colon_l = end_l.with(begin_pos: end_l.end_pos - 1)
 
       [ # modified end token
         [ value(end_t), quote_l ],
@@ -1339,6 +1676,18 @@ module Parser
       Source::Map::Operator.new(loc(op_t), expr_l)
     end
 
+    def range_map(start_e, op_t, end_e)
+      if start_e && end_e
+        expr_l = join_exprs(start_e, end_e)
+      elsif start_e
+        expr_l = start_e.loc.expression.join(loc(op_t))
+      elsif end_e
+        expr_l = loc(op_t).join(end_e.loc.expression)
+      end
+
+      Source::Map::Operator.new(loc(op_t), expr_l)
+    end
+
     def arg_prefix_map(op_t, name_t=nil)
       if name_t.nil?
         expr_l = loc(op_t)
@@ -1351,9 +1700,7 @@ module Parser
 
     def kwarg_map(name_t, value_e=nil)
       label_range = loc(name_t)
-      name_range  = Source::Range.new(label_range.source_buffer,
-                                      label_range.begin_pos,
-                                      label_range.end_pos - 1)
+      name_range  = label_range.adjust(end_pos: -1)
 
       if value_e
         expr_l = loc(name_t).join(value_e.loc.expression)
@@ -1424,6 +1771,11 @@ module Parser
                             expr_l)
     end
 
+    def index_map(receiver_e, lbrack_t, rbrack_t)
+      Source::Map::Index.new(loc(lbrack_t), loc(rbrack_t),
+                             receiver_e.loc.expression.join(loc(rbrack_t)))
+    end
+
     def send_index_map(receiver_e, lbrack_t, rbrack_t)
       Source::Map::Send.new(nil, loc(lbrack_t).join(loc(rbrack_t)),
                             nil, nil,
@@ -1529,6 +1881,13 @@ module Parser
                                  begin_l.join(end_l))
     end
 
+    def guard_map(keyword_t, guard_body_e)
+      keyword_l = loc(keyword_t)
+      guard_body_l = guard_body_e.loc.expression
+
+      Source::Map::Keyword.new(keyword_l, nil, nil, keyword_l.join(guard_body_l))
+    end
+
     #
     # HELPERS
     #
@@ -1557,19 +1916,17 @@ module Parser
       source = static_string(parts)
       return nil if source.nil?
 
-      if defined?(Encoding)
-        source = case
-        when options.children.include?(:u)
-          source.encode(Encoding::UTF_8)
-        when options.children.include?(:e)
-          source.encode(Encoding::EUC_JP)
-        when options.children.include?(:s)
-          source.encode(Encoding::WINDOWS_31J)
-        when options.children.include?(:n)
-          source.encode(Encoding::BINARY)
-        else
-          source
-        end
+      source = case
+      when options.children.include?(:u)
+        source.encode(Encoding::UTF_8)
+      when options.children.include?(:e)
+        source.encode(Encoding::EUC_JP)
+      when options.children.include?(:s)
+        source.encode(Encoding::WINDOWS_31J)
+      when options.children.include?(:n)
+        source.encode(Encoding::BINARY)
+      else
+        source
       end
 
       Regexp.new(source, (Regexp::EXTENDED if options.children.include?(:x)))
@@ -1591,16 +1948,12 @@ module Parser
       token[0]
     end
 
-    if defined?(Encoding)
-      def string_value(token)
-        unless token[0].valid_encoding?
-          diagnostic(:error, :invalid_encoding, nil, token[1])
-        end
-
-        token[0]
+    def string_value(token)
+      unless token[0].valid_encoding?
+        diagnostic(:error, :invalid_encoding, nil, token[1])
       end
-    else
-      alias string_value value
+
+      token[0]
     end
 
     def loc(token)
