@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module Parser
 
   module Lexer::Explanation
@@ -19,14 +21,14 @@ module Parser
       more = "(in-kwarg)" if @in_kwarg
 
       puts decorate(range,
-                    "\e[0;32m#{type} #{val.inspect}\e[0m",
-                    "#{state.to_s.ljust(12)} #{@cond} #{@cmdarg} #{more}\e[0m")
+                    Color.green("#{type} #{val.inspect}"),
+                    "#{state.to_s.ljust(12)} #{@cond} #{@cmdarg} #{more}")
 
       [ type, [val, range] ]
     end
 
     def state=(new_state)
-      puts "  \e[1;33m>>> STATE SET <<<\e[0m " +
+      puts "  #{Color.yellow(">>> STATE SET <<<", bold: true)} " +
            "#{new_state.to_s.ljust(12)} #{@cond} #{@cmdarg}".rjust(66)
 
       self.state_before_explanation = new_state
@@ -38,12 +40,12 @@ module Parser
       from, to = range.begin.column, range.end.column
 
       line = range.source_line + '   '
-      line[from...to] = "\e[4m#{line[from...to]}\e[0m"
+      line[from...to] = Color.underline(line[from...to])
 
       tail_len   = to - from - 1
       tail       = '~' * (tail_len >= 0 ? tail_len : 0)
-      decoration =  "#{" " * from}\e[1;31m^#{tail}\e[0m #{token} ".
-                        ljust(70) + info
+      decoration =  "#{" " * from}#{Color.red("^#{tail}", bold: true)} #{token} ".
+                        ljust(68) + info
 
       [ line, decoration ]
     end
