@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module Parser
   # Parser metadata
   module Meta
@@ -16,11 +18,19 @@ module Parser
         match_with_lvasgn match_current_line
         module class sclass def defs undef alias args
         cbase arg optarg restarg blockarg block_pass kwarg kwoptarg
-        kwrestarg send csend super zsuper yield block
+        kwrestarg kwnilarg send csend super zsuper yield block
         and not or if when case while until while_post
         until_post for break next redo return resbody
         kwbegin begin retry preexe postexe iflipflop eflipflop
-        shadowarg complex rational __FILE__ __LINE__
+        shadowarg complex rational __FILE__ __LINE__ __ENCODING__
+        ident root lambda indexasgn index procarg0
+        restarg_expr blockarg_expr
+        objc_kwarg objc_restarg objc_varargs
+        numargs numblock forward_args forwarded_args
+        case_match in_match in_pattern
+        match_var pin match_alt match_as match_rest
+        array_pattern match_with_trailing_comma array_pattern_with_tail
+        hash_pattern const_pattern if_guard unless_guard match_nil_pattern
       ).map(&:to_sym).to_set.freeze
 
   end # Meta
