@@ -1,18 +1,30 @@
+# frozen_string_literal: true
+
 require 'tempfile'
 require 'minitest/test'
 
 require 'simplecov'
 
 if ENV.include?('COVERAGE') && SimpleCov.usable?
-  if defined?(TracePoint)
-    require_relative 'racc_coverage_helper'
+  require_relative 'racc_coverage_helper'
 
-    RaccCoverage.start(%w(ruby18.y ruby19.y ruby20.y ruby21.y ruby22.y ruby23.y ruby24.y ruby25.y),
-                       File.expand_path('../../lib/parser', __FILE__))
+  RaccCoverage.start(
+    %w(
+      ruby18.y
+      ruby19.y
+      ruby20.y
+      ruby21.y
+      ruby22.y
+      ruby23.y
+      ruby24.y
+      ruby25.y
+      ruby26.y
+      ruby27.y
+    ),
+    File.expand_path('../../lib/parser', __FILE__))
 
-    # Report results faster.
-    at_exit { RaccCoverage.stop }
-  end
+  # Report results faster.
+  at_exit { RaccCoverage.stop }
 
   SimpleCov.start do
     self.formatter = SimpleCov::Formatter::MultiFormatter[
@@ -39,3 +51,10 @@ require 'minitest/autorun'
 
 $LOAD_PATH.unshift(File.expand_path('../../lib', __FILE__))
 require 'parser'
+
+class Parser::AST::Node
+  def initialize(type, *)
+    raise "Type #{type} missing from Parser::Meta::NODE_TYPES" unless Parser::Meta::NODE_TYPES.include?(type)
+    super
+  end
+end
