@@ -1,17 +1,13 @@
+# frozen_string_literal: true
+
 module ParseHelper
   include AST::Sexp
 
-  if RUBY_VERSION == '1.8.7'
-    require 'parser/ruby18'
+  require 'parser/all'
+  require 'parser/macruby'
+  require 'parser/rubymotion'
 
-    ALL_VERSIONS = %w(1.8)
-  else
-    require 'parser/all'
-    require 'parser/macruby'
-    require 'parser/rubymotion'
-
-    ALL_VERSIONS = %w(1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 mac ios)
-  end
+  ALL_VERSIONS = %w(1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 mac ios)
 
   def setup
     @diagnostics = []
@@ -29,6 +25,8 @@ module ParseHelper
     when '2.3' then parser = Parser::Ruby23.new
     when '2.4' then parser = Parser::Ruby24.new
     when '2.5' then parser = Parser::Ruby25.new
+    when '2.6' then parser = Parser::Ruby26.new
+    when '2.7' then parser = Parser::Ruby27.new
     when 'mac' then parser = Parser::MacRuby.new
     when 'ios' then parser = Parser::RubyMotion.new
     else raise "Unrecognized Ruby version #{version}"
@@ -126,6 +124,12 @@ module ParseHelper
 
       assert_source_range(begin_pos, end_pos, range, version, line.inspect)
     end
+
+    assert parser.instance_eval { @lexer }.cmdarg.empty?,
+      "(#{version}) expected cmdarg to be empty after parsing"
+
+    assert_equal 0, parser.instance_eval { @lexer.instance_eval { @paren_nest } },
+      "(#{version}) expected paren_nest to be 0 after parsing"
   end
 
   # Use like this:
@@ -185,6 +189,42 @@ module ParseHelper
     end
   end
 
+  # Use like this:
+  # ~~~
+  # assert_diagnoses_many(
+  #   [
+  #     [:warning, :ambiguous_literal],
+  #     [:error, :unexpected_token, { :token => :tLCURLY }]
+  #   ],
+  #   %q{m /foo/ {}},
+  #   SINCE_2_4)
+  # ~~~
+  def assert_diagnoses_many(diagnostics, code, versions=ALL_VERSIONS)
+    with_versions(versions) do |version, parser|
+      source_file = Parser::Source::Buffer.new('(assert_diagnoses_many)')
+      source_file.source = code
+
+      begin
+        parser = parser.parse(source_file)
+      rescue Parser::SyntaxError
+        # do nothing; the diagnostic was reported
+      end
+
+      assert_equal diagnostics.count, @diagnostics.count
+
+      diagnostics.zip(@diagnostics) do |expected_diagnostic, actual_diagnostic|
+        level, reason, arguments = expected_diagnostic
+        arguments ||= {}
+        message     = Parser::MESSAGES[reason] % arguments
+
+        assert_equal level, actual_diagnostic.level
+        assert_equal reason, actual_diagnostic.reason
+        assert_equal arguments, actual_diagnostic.arguments
+        assert_equal message, actual_diagnostic.message
+      end
+    end
+  end
+
   def refute_diagnoses(code, versions=ALL_VERSIONS)
     with_versions(versions) do |version, parser|
       source_file = Parser::Source::Buffer.new('(refute_diagnoses)')
@@ -202,6 +242,21 @@ module ParseHelper
     end
   end
 
+  def assert_context(context, code, versions=ALL_VERSIONS)
+    with_versions(versions) do |version, parser|
+      source_file = Parser::Source::Buffer.new('(assert_context)')
+      source_file.source = code
+
+      begin
+        parser.parse(source_file)
+      rescue Parser::SyntaxError
+        # do nothing; the diagnostic was reported
+      end
+
+      assert_equal parser.context.stack, context, "(#{version}) parsing context"
+    end
+  end
+
   SOURCE_MAP_DESCRIPTION_RE =
       /(?x)
        ^(?# $1 skip)            ^(\s*)
